@@ -13,38 +13,43 @@
 //! is placed.  The system that places these values there is called a
 //! [`Sink`].
 //!
-//! To implement a sink you need to get a [`SlotWrapper`] for your own crate.
-//! Such a slot can be created with the [`make_slot_wrapper`] macro.  This
-//! wrapper acts as a newtype around an `Option<T>` and derefs into an
-//! `Option<T>`.  To use it implement your desired [`Sink`] for it.
+//! While sinks can be implemented on arbitrary types it's more typical to
+//! implement them on a [`SlotWrapper`].  Due to Rust's orphan rules you need
+//! to create your own [`SlotWrapper`] type in your crate by using the
+//! [`make_slot_wrapper`](crate::make_slot_wrapper`) macro ([more
+//! information](https://doc.rust-lang.org/error-index.html#E0117)).
 //!
-//! Each crate must create its own slot wrapper as it's otherwise not
-//! possible to implement a [`Sink`] for it due to [Rust's orphan
-//! rules](https://doc.rust-lang.org/error-index.html#E0117).
+//! A [`SlotWrapper`] acts as a newtype around an `Option<T>` and derefs into an
+//! `Option<T>`.  To use it implement your desired [`Sink`] for it.  This has the
+//! advantage that it does not need to be allocated.  By calling
+//! [`SlotWrapper::make_handle`] on a slot, one can directly retrieve a
+//! [`SinkHandle`] without the need to box up the slot.
 //!
 //! # Driver
 //!
 //! Because the serialization interface of `deser` is tricky to use with lifetimes
 //! without using a lot of stack space, a safe abstraction is provided with the
-//! [`Driver`] and [`OwnedDriver`] types which allow you to drive the
-//! deserialization process without using stack space.  You feed it events and
-//! internally the driver ensures that the deserlization system is driven in the
-//! right way.
+//! [`Driver`] type which allow you to drive the deserialization process without
+//! using stack space.  You feed it events and internally the driver ensures
+//! that the deserlization system is driven in the right way.
 //!
 //! ```rust
 //! use std::collections::BTreeMap;
-//! use deser::de::OwnedDriver;
+//! use deser::de::Driver;
 //! use deser::Event;
 //!
-//! let mut driver = OwnedDriver::<BTreeMap<u32, String>>::new();
-//! driver.emit(&Event::MapStart).unwrap();
-//! driver.emit(&Event::I64(1)).unwrap();
-//! driver.emit(&Event::Str("Hello".into())).unwrap();
-//! driver.emit(&Event::I64(2)).unwrap();
-//! driver.emit(&Event::Str("World".into())).unwrap();
-//! driver.emit(&Event::MapEnd).unwrap();
+//! let mut out = None::<BTreeMap<u32, String>>;
+//! {
+//!     let mut driver = Driver::new(&mut out);
+//!     driver.emit(&Event::MapStart).unwrap();
+//!     driver.emit(&Event::I64(1)).unwrap();
+//!     driver.emit(&Event::Str("Hello".into())).unwrap();
+//!     driver.emit(&Event::I64(2)).unwrap();
+//!     driver.emit(&Event::Str("World".into())).unwrap();
+//!     driver.emit(&Event::MapEnd).unwrap();
+//! }
 //!
-//! let map = driver.finish();
+//! let map = out.unwrap();
 //! assert_eq!(map[&1], "Hello");
 //! assert_eq!(map[&2], "World");
 //! ```
@@ -57,7 +62,7 @@
 //! must be placed in the slot:
 //!
 //! ```rust
-//! use deser::de::{Sink, Deserializable, DeserializerState};
+//! use deser::de::{Sink, Deserializable, DeserializerState, SinkHandle};
 //! use deser::{make_slot_wrapper, Error};
 //!
 //! make_slot_wrapper!(SlotWrapper);
@@ -78,10 +83,10 @@
 //! }
 //!
 //! impl Deserializable for MyBool {
-//!     fn attach(out: &mut Option<Self>) -> &mut dyn Sink {
-//!         // create your intended slot wrapper here and have it wrap
-//!         // the original slot.
-//!         SlotWrapper::wrap(out)
+//!     fn deserialize_into(out: &mut Option<Self>) -> SinkHandle {
+//!         // Since we're using the SlotWrapper abstraction we can directly
+//!         // make a handle here by using the `make_handle` utility.
+//!         SlotWrapper::make_handle(out)
 //!     }
 //! }
 //! ```
@@ -92,7 +97,7 @@
 //! [`MapSink`] and return it from the main [`Sink`]:
 //!
 //! ```rust
-//! use deser::de::{DeserializerState, Deserializable, Sink, MapSink, ignore};
+//! use deser::de::{DeserializerState, Deserializable, Sink, SinkHandle, MapSink, ignore};
 //! use deser::{make_slot_wrapper, Error, ErrorKind};
 //!
 //! make_slot_wrapper!(SlotWrapper);
@@ -103,9 +108,8 @@
 //! }
 //!
 //! impl Deserializable for Flag {
-//!     fn attach(out: &mut Option<Self>) -> &mut dyn Sink {
-//!         // create your intended slot wrapper here
-//!         SlotWrapper::wrap(out)
+//!     fn deserialize_into(out: &mut Option<Self>) -> SinkHandle {
+//!         SlotWrapper::make_handle(out)
 //!     }
 //! }
 //!
@@ -117,7 +121,7 @@
 //!         // return a new map sink for our struct
 //!         Ok(Box::new(FlagMapSink {
 //!             // note that we can directly connect our slot wrapper
-//!             // to the output slot on the sink.
+//!             // to the output slot on the sink as it deref's into an Option
 //!             out: self,
 //!             key: None,
 //!             enabled_field: None,
@@ -134,25 +138,25 @@
 //! }
 //!     
 //! impl<'a> MapSink for FlagMapSink<'a> {
-//!     fn key(&mut self) -> Result<&mut dyn Sink, Error> {
+//!     fn key(&mut self) -> Result<SinkHandle, Error> {
 //!         // directly attach to the key field which can hold any
 //!         // string value.  This means that any string is accepted
 //!         // as key.
-//!         Ok(Deserializable::attach(&mut self.key))
+//!         Ok(Deserializable::deserialize_into(&mut self.key))
 //!     }
 //!     
-//!     fn value(&mut self) -> Result<&mut dyn Sink, Error> {
+//!     fn value(&mut self) -> Result<SinkHandle, Error> {
 //!         // whenever we are looking for a value slot, look at the last key
 //!         // to decide which value slot to connect.
 //!         match self.key.take().as_deref() {
-//!             Some("enabled") => Ok(Deserializable::attach(&mut self.enabled_field)),
-//!             Some("name") => Ok(Deserializable::attach(&mut self.name_field)),
+//!             Some("enabled") => Ok(Deserializable::deserialize_into(&mut self.enabled_field)),
+//!             Some("name") => Ok(Deserializable::deserialize_into(&mut self.name_field)),
 //!             // if we don't know the key, return a ignore sink to drop the value.
-//!             _ => Ok(ignore()),
+//!             _ => Ok(SinkHandle::to(ignore())),
 //!         }
 //!     }
 //!     
-//!     fn finish(&mut self) -> Result<(), Error> {
+//!     fn finish(&mut self, _state: &DeserializerState) -> Result<(), Error> {
 //!         // when we're done, write the final value into the output slot.
 //!         *self.out = Some(Flag {
 //!             enabled: self.enabled_field.take().ok_or_else(|| {
@@ -169,8 +173,8 @@
 use std::borrow::Cow;
 use std::cell::{Ref, RefMut};
 use std::fmt;
-use std::mem::ManuallyDrop;
-use std::ptr::NonNull;
+use std::mem::{replace, ManuallyDrop};
+use std::ops::{Deref, DerefMut};
 
 use crate::descriptors::{Descriptor, NullDescriptor};
 use crate::error::{Error, ErrorKind};
@@ -184,10 +188,68 @@ pub use self::ignore::ignore;
 
 __make_slot_wrapper!((pub), SlotWrapper);
 
+/// A handle to a [`Sink`].
+///
+/// During deserialization the sinks often need to return other sinks
+/// to recurse into structures.  This poses a challenge if the target
+/// sink cannot be directly borrowed.  This is where [`SinkHandle`]
+/// comes in.  In cases where the [`Sink`] cannot be borrowed it can
+/// be boxed up inside the handle.
+///
+/// The equivalent for serialization is the
+/// [`SerializableHandle`](crate::ser::SerializableHandle).
+pub enum SinkHandle<'a> {
+    /// A borrowed reference to a [`Sink`].
+    Borrowed(&'a mut dyn Sink),
+    /// A boxed up [`Sink`] within the handle.
+    Owned(Box<dyn Sink + 'a>),
+}
+
+impl<'a> Deref for SinkHandle<'a> {
+    type Target = dyn Sink + 'a;
+
+    fn deref(&self) -> &Self::Target {
+        match self {
+            SinkHandle::Borrowed(val) => &**val,
+            SinkHandle::Owned(val) => &**val,
+        }
+    }
+}
+
+impl<'a> DerefMut for SinkHandle<'a> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        match self {
+            SinkHandle::Borrowed(val) => &mut **val,
+            SinkHandle::Owned(val) => &mut **val,
+        }
+    }
+}
+
+impl<'a> SinkHandle<'a> {
+    /// Create a borrowed handle to a [`Sink`].
+    pub fn to(val: &'a mut dyn Sink) -> SinkHandle<'a> {
+        SinkHandle::Borrowed(val)
+    }
+
+    /// Create an owned handle to a heap allocated [`Sink`].
+    pub fn boxed<S: Sink + 'a>(val: S) -> SinkHandle<'a> {
+        SinkHandle::Owned(Box::new(val))
+    }
+}
+
 /// A trait for deserializable types.
+///
+/// A type is deserializable if it can deserialize into a [`Sink`].  The
+/// actual deserialization logic itself is implemented by the returned
+/// [`Sink`].
 pub trait Deserializable: Sized {
     /// Creates a sink that deserializes the value into the given slot.
-    fn attach(out: &mut Option<Self>) -> &mut dyn Sink;
+    ///
+    /// There are two typical implementations for this method: the common one is
+    /// to return a [`SlotWrapper`].  Custom types will most likely just return
+    /// that.  An alternative method is to "wrap" the deserializable in a custom
+    /// sink.
+    fn deserialize_into(out: &mut Option<Self>) -> SinkHandle;
 
     /// Internal method to specialize byte arrays.
     #[doc(hidden)]
@@ -293,16 +355,16 @@ pub trait MapSink {
     }
 
     /// Produces the [`Sink`] for the next key.
-    fn key(&mut self) -> Result<&mut dyn Sink, Error>;
+    fn key(&mut self) -> Result<SinkHandle, Error>;
 
     /// Produces the [`Sink`] for the next value.
     ///
     /// This can inspect the last key to make a decision about which
     /// sink to produce.
-    fn value(&mut self) -> Result<&mut dyn Sink, Error>;
+    fn value(&mut self) -> Result<SinkHandle, Error>;
 
     /// Called when all pairs were produced.
-    fn finish(&mut self) -> Result<(), Error>;
+    fn finish(&mut self, _state: &DeserializerState) -> Result<(), Error>;
 }
 
 /// A trait to produce sinks for items in a sequence.
@@ -313,16 +375,16 @@ pub trait SeqSink {
     }
 
     /// Produces the [`Sink`] for the next item.
-    fn item(&mut self) -> Result<&mut dyn Sink, Error>;
+    fn item(&mut self) -> Result<SinkHandle, Error>;
 
     /// Called when all items were produced.
-    fn finish(&mut self) -> Result<(), Error>;
+    fn finish(&mut self, _state: &DeserializerState) -> Result<(), Error>;
 }
 
 /// Gives access to the deserializer state.
 pub struct DeserializerState<'a> {
     extensions: Extensions,
-    stack: ManuallyDrop<Vec<(SinkHandle, Layer<'a>)>>,
+    stack: ManuallyDrop<Vec<(SinkHandleWrapper, Layer<'a>)>>,
 }
 
 impl<'a> DeserializerState<'a> {
@@ -360,19 +422,18 @@ impl<'a> DeserializerState<'a> {
 /// hides the unsafety internally.
 pub struct Driver<'a> {
     state: ManuallyDrop<DeserializerState<'a>>,
-    current_sink: SinkHandle,
+    current_sink: SinkHandleWrapper,
 }
 
-#[derive(Copy, Clone)]
-struct SinkHandle {
-    sink: NonNull<dyn Sink>,
+struct SinkHandleWrapper {
+    sink: SinkHandle<'static>,
     used: bool,
 }
 
-impl SinkHandle {
-    unsafe fn from(sink: &dyn Sink) -> SinkHandle {
-        SinkHandle {
-            sink: extend_lifetime!(NonNull::from(sink), NonNull<dyn Sink>),
+impl SinkHandleWrapper {
+    unsafe fn from<'a>(sink: SinkHandle<'a>) -> SinkHandleWrapper {
+        SinkHandleWrapper {
+            sink: extend_lifetime!(sink, SinkHandle<'_>),
             used: false,
         }
     }
@@ -386,12 +447,17 @@ enum Layer<'a> {
 impl<'a> Driver<'a> {
     /// Creates a new deserializer driver.
     pub fn new<T: Deserializable>(out: &'a mut Option<T>) -> Driver<'a> {
+        Driver::from_sink(T::deserialize_into(out))
+    }
+
+    /// Creates a new deserializer driver from a sink.
+    pub fn from_sink(sink: SinkHandle) -> Driver<'a> {
         Driver {
             state: ManuallyDrop::new(DeserializerState {
                 extensions: Extensions::default(),
                 stack: ManuallyDrop::new(Vec::new()),
             }),
-            current_sink: unsafe { SinkHandle::from(T::attach(out)) },
+            current_sink: unsafe { SinkHandleWrapper::from(sink) },
         }
     }
 
@@ -415,7 +481,7 @@ impl<'a> Driver<'a> {
             "cannot emit event because sink has already been used"
         );
 
-        let current_sink = unsafe { self.current_sink.sink.as_mut() };
+        let current_sink = &mut self.current_sink.sink;
         match event {
             Event::Null => current_sink.null(&self.state)?,
             Event::Bool(v) => current_sink.bool(*v, &self.state)?,
@@ -426,33 +492,30 @@ impl<'a> Driver<'a> {
             Event::F64(v) => current_sink.f64(*v, &self.state)?,
             Event::MapStart => {
                 let mut map_sink = current_sink.map(&self.state)?;
-                let old_sink = self.current_sink;
-                self.current_sink = unsafe { SinkHandle::from(map_sink.key()?) };
-                self.state.stack.push((old_sink, unsafe {
-                    extend_lifetime!(Layer::Map(map_sink, true), Layer<'_>)
-                }));
+                let key_sink = unsafe { SinkHandleWrapper::from(map_sink.key()?) };
+                let layer = unsafe { extend_lifetime!(Layer::Map(map_sink, true), Layer<'_>) };
+                let old_sink = replace(&mut self.current_sink, key_sink);
+                self.state.stack.push((old_sink, layer));
                 return Ok(());
             }
             Event::MapEnd => match self.state.stack.pop() {
                 Some((next_sink, Layer::Map(mut map_sink, _))) => {
-                    map_sink.finish()?;
+                    map_sink.finish(&self.state)?;
                     self.current_sink = next_sink;
                 }
                 _ => panic!("not inside a MapSink"),
             },
             Event::SeqStart => {
                 let mut seq_sink = current_sink.seq(&self.state)?;
-                let item_sink = seq_sink.item()?;
-                let old_sink = self.current_sink;
-                self.current_sink = unsafe { SinkHandle::from(item_sink) };
-                self.state.stack.push((old_sink, unsafe {
-                    extend_lifetime!(Layer::Seq(seq_sink), Layer<'_>)
-                }));
+                let item_sink = unsafe { SinkHandleWrapper::from(seq_sink.item()?) };
+                let layer = unsafe { extend_lifetime!(Layer::Seq(seq_sink), Layer<'_>) };
+                let old_sink = replace(&mut self.current_sink, item_sink);
+                self.state.stack.push((old_sink, layer));
                 return Ok(());
             }
             Event::SeqEnd => match self.state.stack.pop() {
                 Some((next_sink, Layer::Seq(mut seq_sink))) => {
-                    seq_sink.finish()?;
+                    seq_sink.finish(&self.state)?;
                     self.current_sink = next_sink;
                 }
                 _ => panic!("not inside a SeqSink"),
@@ -469,10 +532,10 @@ impl<'a> Driver<'a> {
                     map_sink.key()?
                 };
                 *is_key = !*is_key;
-                self.current_sink = unsafe { SinkHandle::from(next_sink) };
+                self.current_sink = unsafe { SinkHandleWrapper::from(next_sink) };
             }
             Some((_, Layer::Seq(ref mut seq_sink))) => {
-                self.current_sink = unsafe { SinkHandle::from(seq_sink.item()?) };
+                self.current_sink = unsafe { SinkHandleWrapper::from(seq_sink.item()?) };
             }
             _ => {}
         }
@@ -494,68 +557,6 @@ impl<'a> Drop for Driver<'a> {
     }
 }
 
-/// An alternative version of [`Driver`] that owns the value.
-pub struct OwnedDriver<T> {
-    slot: Box<Option<T>>,
-    // the static lifetime is a lie.  It's in fact borrowed
-    // into the `slot` box.
-    driver: ManuallyDrop<Driver<'static>>,
-}
-
-impl<T: Deserializable + 'static> Default for OwnedDriver<T> {
-    fn default() -> Self {
-        OwnedDriver::new()
-    }
-}
-
-impl<T: Deserializable + 'static> OwnedDriver<T> {
-    /// Creates a new owned driver.
-    pub fn new() -> OwnedDriver<T> {
-        let mut slot = Box::new(None::<T>);
-        let driver = Driver::new(unsafe { extend_lifetime!(&mut slot, &mut Box<Option<T>>) });
-        OwnedDriver {
-            slot,
-            driver: ManuallyDrop::new(unsafe { extend_lifetime!(driver, Driver<'_>) }),
-        }
-    }
-
-    /// Emits an event into the driver.
-    ///
-    /// For more information see [`Driver::emit`] which has the same behavior.
-    pub fn emit(&mut self, event: &Event) -> Result<(), Error> {
-        self.driver.emit(event)
-    }
-
-    /// Returns `true` if the driver holds a value.
-    pub fn is_finished(&self) -> bool {
-        self.slot.is_some()
-    }
-
-    /// Finishes the driver and returns the produced value.
-    ///
-    /// # Panics
-    ///
-    /// Unlike the regular [`Driver`] and [`OwnedDriver`] panics if the driver
-    /// did not manage to produce a value.  This should not happen unless an
-    /// implementation of [`Deserializable`] returns a sink that ignores all values
-    /// which is not legal to do.  This method will not panic if
-    /// [`is_finished`](Self::is_finished) returns `true`.
-    pub fn finish(mut self) -> T {
-        self.slot
-            .take()
-            .expect("driver finished without producing value")
-    }
-}
-
-impl<T> Drop for OwnedDriver<T> {
-    fn drop(&mut self) {
-        // make sure to first drop the driver before the slot goes away.
-        unsafe {
-            ManuallyDrop::drop(&mut self.driver);
-        }
-    }
-}
-
 #[test]
 fn test_driver() {
     let mut out: Option<std::collections::BTreeMap<u32, String>> = None;
