@@ -4,7 +4,9 @@ use std::hash::BuildHasher;
 
 use crate::descriptors::{Descriptor, NamedDescriptor, NumberDescriptor, UnorderedNamedDescriptor};
 use crate::error::Error;
-use crate::ser::{Chunk, MapEmitter, SeqEmitter, Serializable, SerializerState};
+use crate::ser::{
+    Chunk, MapEmitter, SeqEmitter, Serializable, SerializableHandle, SerializerState,
+};
 
 impl Serializable for bool {
     fn descriptor(&self) -> &dyn Descriptor {
@@ -254,8 +256,8 @@ where
 struct SliceEmitter<'a, T>(std::slice::Iter<'a, T>);
 
 impl<'a, T: Serializable> SeqEmitter for SliceEmitter<'a, T> {
-    fn next(&mut self) -> Option<&dyn Serializable> {
-        self.0.next().map(|x| x as _)
+    fn next(&mut self) -> Option<SerializableHandle> {
+        self.0.next().map(SerializableHandle::to)
     }
 }
 
@@ -277,15 +279,15 @@ where
             K: Serializable,
             V: Serializable,
         {
-            fn next_key(&mut self) -> Option<&dyn Serializable> {
+            fn next_key(&mut self) -> Option<SerializableHandle> {
                 self.0.next().map(|(k, v)| {
                     self.1 = Some(v);
-                    k as &dyn Serializable
+                    SerializableHandle::to(k)
                 })
             }
 
-            fn next_value(&mut self) -> &dyn Serializable {
-                self.1.unwrap()
+            fn next_value(&mut self) -> SerializableHandle {
+                SerializableHandle::to(self.1.unwrap())
             }
         }
 
@@ -312,15 +314,15 @@ where
             K: Serializable,
             V: Serializable,
         {
-            fn next_key(&mut self) -> Option<&dyn Serializable> {
+            fn next_key(&mut self) -> Option<SerializableHandle> {
                 self.0.next().map(|(k, v)| {
                     self.1 = Some(v);
-                    k as &dyn Serializable
+                    SerializableHandle::to(k)
                 })
             }
 
-            fn next_value(&mut self) -> &dyn Serializable {
-                self.1.unwrap()
+            fn next_value(&mut self) -> SerializableHandle {
+                SerializableHandle::to(self.1.unwrap())
             }
         }
 
@@ -344,8 +346,8 @@ where
         where
             T: Serializable,
         {
-            fn next(&mut self) -> Option<&dyn Serializable> {
-                self.0.next().map(|v| v as _)
+            fn next(&mut self) -> Option<SerializableHandle> {
+                self.0.next().map(SerializableHandle::to)
             }
         }
 
@@ -369,8 +371,8 @@ where
         where
             T: Serializable,
         {
-            fn next(&mut self) -> Option<&dyn Serializable> {
-                self.0.next().map(|v| v as _)
+            fn next(&mut self) -> Option<SerializableHandle> {
+                self.0.next().map(SerializableHandle::to)
             }
         }
 
